@@ -1,5 +1,5 @@
 /* includes.h - Include common files
- *      Copyright (C) 1999 Michael Roth <mroth@nessie.de>
+ *      Copyright (C) 1999 Michael Roth <mroth@gnupg.org>
  *
  * This file is part of pgpgpg.
  *
@@ -50,6 +50,16 @@
 # include <fcntl.h>
 #endif
 
+/* for OS/2 EMX */
+#ifndef HAVE_STRCASECMP
+# define strcasecmp(a,b) stricmp(a,b)
+#endif
+
+#ifndef HAVE_STRNCASECMP
+# define strncasecmp(a,b,c) strnicmp(a,b,c)
+#endif
+
+
 #include <signal.h>
 
 #include "debug.h"
