@@ -1,5 +1,5 @@
 /* main.c - pgpgpg main function
- *      Copyright (C) 1999 Michael Roth <mroth@nessie.de>
+ *      Copyright (C) 1999 Michael Roth <mroth@gnupg.org>
  *
  * This file is part of pgpgpg.
  *
@@ -167,11 +167,26 @@ int main(int argc, char *argv[])
     
     switch(WEXITSTATUS(child_status))
     {
-        case 0: result = 0; break;
-        default: result = PGPERR_UNKNOWN; break;
+        case 0:
+            result = PGPERR_OK;
+            break;
+        
+        case 1:
+            result = PGPERR_DECR_SIGN;
+            fprintf(stderr, "WARNING: Bad signature, doesn't match file contents!\n");
+            break;
+        
+        case 2:
+            result = PGPERR_KEY_NOT_EXIST;
+            fprintf(stderr, "WARNING: Can't find the right public key-- can't check signature integrity.\n");
+            break;
+        
+        default:
+            result = 0; /* FIXME... */
+            break;
     }
     
-    return 0 /*result*/;	/* FIXME... */
+    return result;
 }
 
 
