@@ -1,5 +1,5 @@
 /* pgpopts.c - Parse pgp long options
- *      Copyright (C) 1999 Michael Roth <mroth@nessie.de>
+ *      Copyright (C) 1999 Michael Roth <mroth@gnupg.org>
  *
  * This file is part of pgpgpg.
  *
@@ -25,6 +25,7 @@
 #include "argvfactory.h"
 #include "defaultkey.h"
 #include "mygetopt.h"
+#include "guessarmortype.h"
 
 
 
@@ -357,7 +358,7 @@ static int parse_longopt(Pgpopts *opts, const char *s)
         
         default:
             fprintf(stderr, "BUG: The long option `%s' is currently not supported.\n"
-                            "     Please submit a bug report to <mroth@nessie.de>\n", mapping[i].keyword);
+                            "     Please submit a bug report to <mroth@gnupg.org>\n", mapping[i].keyword);
             return -1;
     }
     
@@ -632,8 +633,16 @@ static void build_output(ArgvFactory *af, Pgpopts *opts)
             else if (opts->args_used != 0)	/* Hint: if this condition is not given, it will result in an error on build_input() */
             {
                 char *s = (char *)malloc((strlen(opts->args[0])+5) * sizeof(char));
+                
                 strcpy(s, opts->args[0]);
-                strcat(s, opts->a || opts->asciiarmor ? ".asc" : ".pgp" );
+                
+                if (opts->a || opts->asciiarmor)	/* ascii armored output */
+                    strcat(s, ".asc");
+                else if (opts->s && opts->b)		/* detached sign */
+                    strcat(s, ".sig");
+                else					/* binary output */
+                    strcat(s, ".pgp");
+                
                 argv_factory_add(af, "--output");
                 argv_factory_add(af, s);
             }
@@ -668,11 +677,6 @@ static void build_passfd_pgpbug(ArgvFactory *af, Pgpopts *opts)
 
 static void build_keyrings(ArgvFactory *af, Pgpopts *opts)
 {
-#if 0
-    if (opts->pubring || opts->secring)
-        argv_factory_add(af, "--no-default-keyring");
-#endif
-    
     if (opts->pubring)
     {
         argv_factory_add(af, "--no-default-keyring");
@@ -690,13 +694,24 @@ static void build_keyrings(ArgvFactory *af, Pgpopts *opts)
 
 
 
-/*
- * Build gpg arguments from pgp options.
+/*****************************************************************************
  *
- * Please note, PGP contains a lot of bugs in the command line option
- * parsing. However, many tools and scripts which use PGP assume this
- * bugs so we will try to simulate this PGP bugs.
- */
+ * Function : pgpopts_build_argv
+ *
+ * Purpose  : Contructs an argv style array of GnuPG options out of a Pgpopts
+ *            structure.
+ *
+ * Input    : pgpopts - A Pgpopts structure to build the GnuPG options from.
+ *
+ * Output   : An argv style vector or NULL on error.
+ *
+ * Errors   : On en error NULL is returned.
+ *
+ * Notes    : PGP 2.6 contains a lot of bugs in the command line option parsing.
+ *            However, many tools and scripts which use PGP assume this bugs
+ *            so we will try to simulate this PGP bugs.
+ *
+ *****************************************************************************/
 char ** pgpopts_build_argv(Pgpopts *pgpopts)
 {
     Pgpopts	opts;
@@ -933,35 +948,191 @@ char ** pgpopts_build_argv(Pgpopts *pgpopts)
             if (build_input(&af, &opts))
                 return NULL;
         }
-        else						/* decrypt */
+        else						/* decrypt, verify */
         {
             opts.d = 0;	/* FIXME: How to handle these flags?? */
             opts.p = 0;
+            
             build_certdepth(&af, &opts);
-            build_passfd(&af, &opts);
             build_keyrings(&af, &opts);
             
-            if (opts.outputfile)
-            {
-                argv_factory_add(&af, "--output");
-                argv_factory_add(&af, opts.outputfile);
-            }
+            /*
+             * Try to be a little bit intelligent and try to determine if the
+             * user would like to decrypt or verify a message a verify a
+             * detached signature.
+             */
             
-            if (opts.f)
+            if (opts.args_used == 0)
             {
-                opts.f = 0;
-                argv_factory_add(&af, "--decrypt");
-            }
-            else
-            {
-                if (opts.args_used == 0  &&  opts.stdinisterminal)
+                /*
+                 * It's easy: Decrypt or verify a message from stdin
+                 */
+                
+                build_passfd(&af, &opts);
+                
+                if (opts.outputfile)
+                {
+                    argv_factory_add(&af, "--output");
+                    argv_factory_add(&af, opts.outputfile);
+                }
+                
+                if (opts.f)
+                {
+                    opts.f = 0;
+                    argv_factory_add(&af, "--decrypt");
+                }
+                else if (opts.stdinisterminal)
                 {
                     fprintf(stderr, "Error: Filename for decryption needed.\n");
                     return NULL;
                 }
+            }
+            else if (opts.args_used >= 2)
+            {
+                /*
+                 * It's (hopefully) easy: Verify a cleartext signature. But this
+                 *  only works, if the user didn't made a misstake on command line
+                 *  options. The bad thing is, we can't check if the user made a
+                 *  misstake or signature verification failed...
+                 */
                 
-                if (opts.args_used >= 1)
+                argv_factory_add(&af, "--verify");
+                argv_factory_add(&af, opts.args[0]);
+                argv_factory_add(&af, opts.args[1]);
+            }
+            else
+            {
+                /*
+                 * It's complicated: We must try to guess if the user would like 
+                 * to decrypt a file or verify a detached signature. This process
+                 * doesn't work if the detached signature doesn't end with `.sig'
+                 */
+                
+                char *file_base = strdup(opts.args[0]);
+                char *file_ext  = strrchr(file_base, '.');
+                
+                if (file_ext)
+                    *file_ext++ = 0;
+                
+                if (file_ext && !strcasecmp(file_ext, "sig"))
+                {
+                    /*
+                     * Yes, the user would like to check a detached signature.
+                     * We asume the correctly named data file will exists.
+                     */
+                    
+                    argv_factory_add(&af, "--verify");
                     argv_factory_add(&af, opts.args[0]);
+                    argv_factory_add(&af, file_base);
+                }
+                else if(file_ext && !strcasecmp(file_ext, "asc"))
+                {
+                    /*
+                     * The given file is armored data. We try to determine the type of
+                     * armored data. However, this will not work correctly with PGP 2.6
+                     * generated data.
+                     */
+                    
+                    switch (guess_armor_type(opts.args[0])) 
+                    {
+                        case ARMOR_TYPE_SIGNATURE:
+                            /*
+                             * Yes, a detached signature. Assume their is a file to check
+                             * this signature against it, without the .asc extension.
+                             */
+                            argv_factory_add(&af, "--verify");
+                            argv_factory_add(&af, opts.args[0]);
+                            argv_factory_add(&af, file_base);
+                            break;
+                        
+                        case ARMOR_TYPE_SIGNED_MESSAGE:
+                            /*
+                             * This is a clear sign message. Check the signatures.
+                             */
+                            if (opts.outputfile)
+                            {
+                                argv_factory_add(&af, "--output");
+                                argv_factory_add(&af, opts.outputfile);
+                            }
+                            if (opts.f)
+                            {
+                                opts.f = 0;
+                                argv_factory_add(&af, "--decrypt");
+                            }
+                            argv_factory_add(&af, opts.args[0]);
+                            break;
+                        
+                        default:
+                            /*
+                             * This armored data could be either a signed/encrypted or
+                             * a detached signature.
+                             * If a file without the extension .asc exists assume the user
+                             * would like to check a detached signature.
+                             */
+                            if (access(file_base, R_OK)==0)
+                            {
+                                /* Assume a detached signature */
+                                argv_factory_add(&af, "--verify");
+                                argv_factory_add(&af, opts.args[0]);
+                                argv_factory_add(&af, file_base);
+                            }
+                            else
+                            {
+                                if (opts.outputfile)
+                                {
+                                    argv_factory_add(&af, "--output");
+                                    argv_factory_add(&af, opts.outputfile);
+                                }
+                                
+                                if (opts.f) 
+                                {
+                                    opts.f = 0;
+                                    argv_factory_add(&af, "--decrypt");
+                                }
+                                
+                                argv_factory_add(&af, opts.args[0]);
+                                
+                                /* FIXME: should we close stdin ??? */
+                            }
+                    }
+                }
+                else
+                {
+                    /*
+                     * We are not sure what type the input is.
+                     * Hopefully this isn't a renamed detached signature...
+                     */
+                    
+                    int null_fd;
+                    
+                    if (opts.outputfile)
+                    {
+                        argv_factory_add(&af, "--output");
+                        argv_factory_add(&af, opts.outputfile);
+                    }
+                    
+                    if (opts.f)
+                    {
+                        opts.f = 0;
+                        argv_factory_add(&af, "--decrypt");
+                    }
+                    
+                    argv_factory_add(&af, opts.args[0]);
+                    
+                    /*
+                     * Redirect stdin to /dev/null to prevent waiting for data
+                     * on stdin when "decrypting" a detached signature.
+                     */
+                    
+                    if ((null_fd = open("/dev/null", O_RDWR)) == -1)
+                    {
+                        fprintf(stderr, "Can't open /dev/null\n");
+                        return NULL;
+                    }
+                    close(0);
+                    dup2(null_fd, 0);
+                    close(null_fd);
+                }
             }
         }
     }
@@ -995,28 +1166,15 @@ char ** pgpopts_build_argv(Pgpopts *pgpopts)
             build_certdepth(&af, &opts);
             build_output(&af, &opts);
             
-#if 0
-            if (opts.f)
-            {
-                argv_factory_add(&af, "--set-filename");
-                argv_factory_add(&af, "stdin");
-            }
-            else if (opts.args_used == 0)
-            {
-                fprintf(stderr, "Error: Need a filename for public key encryption.\n");
-                return NULL;
-            }
-#endif
-            
             for (i = !opts.f && opts.args_used != 0 ? 1 : 0; opts.args[i]; ++i)
             {
-                argv_factory_add(&af, "--remote-user");
+                argv_factory_add(&af, "--recipient");
                 argv_factory_add(&af, opts.args[i]);
             }
             
             if (opts.encrypttoself)
             {
-                argv_factory_add(&af, "--remote-user");
+                argv_factory_add(&af, "--encrypt-to");
                 if (opts.localuserid)
                     argv_factory_add(&af, opts.localuserid);
                 else
@@ -1031,17 +1189,6 @@ char ** pgpopts_build_argv(Pgpopts *pgpopts)
                 }
             }
             
-#if 0
-            if (!opts.f)
-            {
-                char *s = (char *)malloc((strlen(opts.args[0])+5) * sizeof(char));
-                strcpy(s, opts.args[0]);
-                strcat(s, opts.a || opts.asciiarmor ? ".asc" : ".pgp");
-                argv_factory_add(&af, "--output");
-                argv_factory_add(&af, s);
-            }
-#endif
-            
             if (opts.s)
             {
                 opts.s = 0;
@@ -1054,20 +1201,15 @@ char ** pgpopts_build_argv(Pgpopts *pgpopts)
             if (build_input(&af, &opts))
                 return NULL;
             
-#if 0
-            if (!opts.f)
-                argv_factory_add(&af, opts.args[0]);
-#endif
-            
             opts.f = 0;
         }
         else if (opts.c==0 && opts.e==0 && opts.s==1)	/* sign */
         {
-            opts.s = 0;
-            
             build_localuser(&af, &opts);
             build_output(&af, &opts);
             
+            opts.s = 0; /* build_output() uses this flag too */
+            
             if (opts.clearsign && (opts.a || opts.asciiarmor) && textmode && opts.b==0)
             {
                 argv_factory_add(&af, "--escape-from");
@@ -1214,12 +1356,12 @@ int pgpopts_parse(Pgpopts *pgpopts, int argc, char *argv[])
                 break;
             
             case 'h':
-                printf("Help not supportded.\n");
+                printf("Help not supported.\n");
                 return -1;
                 break;
             
             default:
-                fprintf(stderr, "BUG: Unhandeled option: -%c\n", result);
+                fprintf(stderr, "BUG: Unhandled option: -%c\n", result);
                 return -1;
         }
     
